from classes.User import User
from classes.Transaction import Tx
from classes.TxBlock import TxBlock

from helpers import DatabaseHelper as databaseHelper
from helpers import UtilityHelper as utilityHelper

import socket
import pickle
import select

BUFFER_SIZE = 1024

def newServerSocket(ip_addr, port):
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.bind((ip_addr, port))
    server_socket.listen()
    return server_socket


def recvObj(socket):
    inputs, outputs, errs = select.select([socket], [], [socket], 6)
    if socket in inputs:
        connected_socket, addr = socket.accept()
        all_data = b''
        while True:
            data = connected_socket.recv(BUFFER_SIZE)
            if not data:
                break
            all_data = all_data + data
        return pickle.loads(all_data)
    return None


def sendObj(ip_addr, blk, port):
    try:
        soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        soc.connect((ip_addr, port))
        data = pickle.dumps(blk)
        soc.send(data)
        soc.close()
        return True
    except:
        print("Could not connect to peer")
        return False



def connection(ip, port, db):
    server_socket = newServerSocket(ip, port)
    while(True):

        item = recvObj(server_socket)
        if not item:
            continue

        if type(item) == Tx:
            print(f"{utilityHelper.blinkMessage('Received transaction from peer')}")
            utilityHelper.addFile("../data/transaction_pool.dat", item)
            continue

        if type(item) == TxBlock:
            print(f"{utilityHelper.blinkMessage('Received block from peer')}")
            utilityHelper.addFile("../data/ledger.dat", item)
            continue

        # DATABASE STUFF
        if type(item) == list:
            match item[0]:

                case "USER CREATE":
                    db.createUser(item[1], item[2], item[3], item[4])
                    continue

                case "USER DELETE":
                    db.deleteUser(item[1])
                    continue

                case "USER CHANGE PASSWORD":
                    db.changePassword(item[1], item[2], item[3])
                    continue

                case "USER CHANGE USERNAME":
                    db.changeUsername(item[1], item[2])
                    continue

                case "CREATE LOG":
                    db.createLog(item[1], item[2], item[3])
                    continue

                case "UPDATE LOG STATUS":
                    db.updateLogStatus(item[1])
                    continue

# changes on newServerConnection 
# We modified it to accept port number as a parameter, such that when 
# we make a server we can specify on which port bind it